#ifndef _LINUX_SPI_H
#define _LINUX_SPI_H

#define SEL_SPI_CPOL_0    1
#define SEL_SPI_CPOL_1    2
#define SEL_UWIRE_CPOL_0  3
#define SEL_UWIRE_CPOL_1  4

/* 
 * the value written to ports to test existence. PC-style ports will 
 * return the value written. AT-style ports will return 0. so why not
 * make them the same ? 
 */
#define LP_DUMMY	0x00


#ifndef MODULE
/*
 * function prototypes
 */

extern long spi_init(long);
#endif				/* MODULE */
#endif				/* _LINUX_SPI_H */
